#!/usr/bin/env node

const fs = require('fs-extra')
const path = require('path')
const inquirer = require('inquirer')
const chalk = require('chalk')
const ora = require('ora')
const shell = require('shelljs')
const { templateRootFolder } = require('./config.js')

const log = require('../utils/log.js')

/**
 * 默认的工程脚本提示
 */
exports.defaultProjectTips = (filename) => {
  console.log()
  if (filename) {
    log.cmd(`cd ${filename} && yarn install`)
  } else {
    log.cmd('yarn install')
  }
  log.cmd('yarn run dev')
  log.cmd('yarn run build')
  console.log()
}

/**
 * 判断目录是否为空
 * @param filePath
 * @param fn
 * @returns {boolean}
 */
exports.checkFolder = (filePath, fn) => {
  let cover = false
  fs.readdir(filePath, (error, files) => {
    if (error && error.code !== 'ENOENT') {
      log.error(error.toString(), true)
      process.exit()
    }
    // 确认是否清空
    if (!(!files || !files.length)) {
      inquirer.prompt({
        message: '检测到该文件夹下有文件，是否要清空？',
        type: 'confirm',
        name: 'status',
        default: false
      }).then(res => {
        cover = res.status
        if (cover) {
          const spinner = ora(`正在删除 ${chalk.cyan(filePath.split('/').pop())} 中，请稍等`).start()
          try {
            fs.emptyDirSync(filePath)
          } catch (error) {
            log.error(error)
            process.exit()
          }
          spinner.stop()
        }
        fn()
      })
    } else {
      fn()
    }
  })
  return cover
}

/**
 * 递归获取所有文件并进行操作
 * @param filePath
 * @param handleFile
 * @param handleFolder
 */
exports.fileDisplay = (
  filePath,
  handleFile,
  handleFolder
) => {
  // 根据文件路径读取文件，返回文件列表
  fs.readdir(filePath, (error, files) => {
    if (error) {
      log.error(error.toString(), false)
      process.exit()
    } else {
      // 遍历读取到的文件列表
      files.forEach((filename) => {
        // 获取当前文件的绝对路径
        const file = path.join(filePath, filename)

        // 根据文件路径获取文件信息
        fs.stat(file, (err, stats) => {
          if (err) {
            log.error(err.toString(), false)
            process.exit()
          } else {
            const isFile = stats.isFile()
            const isFolder = stats.isDirectory()
            if (isFile) {
              handleFile(file)
            }
            if (isFolder) {
              fileDisplay(handleFolder(file), handleFile, handleFolder)
            }
          }
        })
      })
    }
  })
}

/**
 * 初始化远程 git 仓库
 */
exports.initGit = async (templateName, projectFolder, fn) => {
  // 检查 git 命令是否存在
  shell.config.silent = true
  try {
    shell.exec('git -h')
  } catch (error) {
    log.error('请先安装 git 命令')
    process.exit()
  }
  // 检查模板根路径是否存在
  if (!fs.existsSync(templateRootFolder)) {
    fs.mkdirSync(templateRootFolder)
  }

  // git 仓库地址
  const gitRemote = `https://github.com/wytxer/${templateName}.git`
  // 本地缓存地址
  const localFolder = path.join(templateRootFolder, templateName)
  // 如果本地不存在模板，重新 clone 下来
  if (!fs.existsSync(localFolder)) {
    console.log()
    console.log('初始化模板中，有点慢请稍等~')
    shell.cd(templateRootFolder)
    try {
      shell.exec(`git clone ${gitRemote}`)
    } catch (error) {
      log.error('模板初始化失败')
      console.error(error)
      process.exit()
    }
  }

  // 到目标目录
  shell.cd(localFolder)
  // 拉取一下
  shell.exec('git pull')

  // 复制到目标目录
  try {
    fs.copySync(localFolder, projectFolder, {
      overwrite: true
    })
    // 删除 .git 文件
    // fs.removeSync(path.join(projectFolder, '/.git/'))
  } catch (error) {
    log.error('模板初始化失败')
    console.error(error)
    process.exit()
  }
  fn()
}

// 异步的 exec，将子进程转成同步进程
exports.exec = (cmd, options = {}) => {
  return new Promise((resolve, reject) => {
    const stdout = shell.exec(cmd, options, (error, stdout, stderr) => {
      if (error) {
        reject(error)
      }
      resolve({
        error,
        stdout,
        stderr
      })
    })
    stdout.on('exit', (code, signal) => {
      resolve({
        code,
        signal
      })
    })
  })
}
